@@ -82,6 +82,7 @@ import com.openexchange.mail.api.MailLogicTools;
 import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.mime.MIMESessionPropertyNames;
+import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.mailaccount.MailAccountException;
 import com.openexchange.mailaccount.MailAccountStorageService;
 import com.openexchange.monitoring.MonitoringInfo;
@@ -459,6 +460,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              */
             try {
                 imapStore.connect(config.getServer(), config.getPort(), login, tmpPass);
+                // final Throwable stack = new Throwable();
             } catch (final AuthenticationFailedException e) {
                 /*
                  * Remember failed authentication's credentials (for a short amount of time) to fasten subsequent connect trials
@@ -470,16 +472,25 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                  * TODO: Re-think if exception's message should be part of condition or just checking if nested exception is an instance of
                  * SocketTimeoutException
                  */
-                if (tmpDownEnabled && SocketTimeoutException.class.isInstance(e.getNextException()) && ((SocketTimeoutException) e.getNextException()).getMessage().toLowerCase(
-                    Locale.ENGLISH).indexOf(ERR_CONNECT_TIMEOUT) != -1) {
-                    /*
-                     * Remember a timed-out IMAP server on connect attempt
-                     */
-                    timedOutServers.put(new HostAndPort(config.getServer(), config.getPort()), Long.valueOf(System.currentTimeMillis()));
+                if (tmpDownEnabled) {
+                    final Exception nextException = e.getNextException();
+                    if (SocketTimeoutException.class.isInstance(nextException) && ((SocketTimeoutException) nextException).getMessage().toLowerCase(
+                        Locale.ENGLISH).indexOf(ERR_CONNECT_TIMEOUT) != -1) {
+                        /*
+                         * Remember a timed-out IMAP server on connect attempt
+                         */
+                        timedOutServers.put(new HostAndPort(config.getServer(), config.getPort()), Long.valueOf(System.currentTimeMillis()));
+                    }
                 }
                 throw e;
             }
             connected = true;
+            /*
+             * Propagate client IP address in case of primary mail account access
+             */
+            if (imapConfProps.isPropagateClientIPAddress() && MailAccount.DEFAULT_ID == accountId) {
+                IMAPCommandsCollection.propagateClientIP((IMAPFolder) imapStore.getFolder("INBOX"), session.getLocalIp());
+            }
             /*
              * Add server's capabilities
              */
@@ -513,7 +524,8 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
     }
 
     private void checkTemporaryDown(final IIMAPProperties imapConfProps) throws MailException, IMAPException {
-        final HostAndPort key = new HostAndPort(getMailConfig().getServer(), getMailConfig().getPort());
+        final MailConfig mailConfig = getMailConfig();
+        final HostAndPort key = new HostAndPort(mailConfig.getServer(), mailConfig.getPort());
         final Map<HostAndPort, Long> map = timedOutServers;
         final Long range = map.get(key);
         if (range != null) {
@@ -521,7 +533,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 /*
                  * Still treated as being temporary broken
                  */
-                throw IMAPException.create(IMAPException.Code.CONNECT_ERROR, getMailConfig().getServer(), getMailConfig().getLogin());
+                throw IMAPException.create(IMAPException.Code.CONNECT_ERROR, mailConfig.getServer(), mailConfig.getLogin());
             }
             map.remove(key);
         }
