@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -76,6 +76,7 @@ import com.openexchange.imap.acl.ACLExtensionFactory;
 import com.openexchange.imap.cache.MBoxEnabledCache;
 import com.openexchange.imap.cache.NamespaceFoldersCache;
 import com.openexchange.imap.cache.RightsCache;
+import com.openexchange.imap.cache.RootSubfolderCache;
 import com.openexchange.imap.cache.UserFlagsCache;
 import com.openexchange.imap.command.CopyIMAPCommand;
 import com.openexchange.imap.command.FlagsIMAPCommand;
@@ -467,22 +468,28 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                         IMAPException.Code.FOLDER_DOES_NOT_HOLD_FOLDERS,
                         imapConfig,
                         session,
-                        parent instanceof DefaultFolder ? DEFAULT_FOLDER_ID : parentFullname);
+                        isParentDefault ? DEFAULT_FOLDER_ID : parentFullname);
                 }
                 /*
                  * Check ACLs if enabled
                  */
                 if (imapConfig.isSupportsACLs()) {
                     try {
-                        if (!getACLExtension().canCreate(RightsCache.getCachedRights(parent, true, session, accountId))) {
-                            throw IMAPException.create(IMAPException.Code.NO_CREATE_ACCESS, imapConfig, session, parentFullname);
+                        if (isParentDefault) {
+                            if (!(RootSubfolderCache.canCreateSubfolders((DefaultFolder) parent, true, session, accountId).booleanValue())) {
+                                throw IMAPException.create(IMAPException.Code.NO_CREATE_ACCESS, imapConfig, session, DEFAULT_FOLDER_ID);
+                            }
+                        } else {
+                            if (!getACLExtension().canCreate(RightsCache.getCachedRights(parent, true, session, accountId))) {
+                                throw IMAPException.create(IMAPException.Code.NO_CREATE_ACCESS, imapConfig, session, parentFullname);
+                            }
                         }
                     } catch (final MessagingException e) {
                         /*
                          * MYRIGHTS command failed for given mailbox
                          */
                         if (!imapConfig.getImapCapabilities().hasNamespace() || !NamespaceFoldersCache.containedInPersonalNamespaces(
-                            parent.getFullName(),
+                            parentFullname,
                             imapStore,
                             true,
                             session,
@@ -490,7 +497,12 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                             /*
                              * No namespace support or given parent is NOT covered by user's personal namespaces.
                              */
-                            throw IMAPException.create(IMAPException.Code.NO_ACCESS, imapConfig, session, e, parentFullname);
+                            throw IMAPException.create(
+                                IMAPException.Code.NO_ACCESS,
+                                imapConfig,
+                                session,
+                                e,
+                                isParentDefault ? DEFAULT_FOLDER_ID : parentFullname);
                         }
                         if (DEBUG) {
                             LOG.debug("MYRIGHTS command failed on namespace folder", e);
@@ -541,7 +553,7 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                                 imapConfig,
                                 session,
                                 createMe.getFullName(),
-                                parent instanceof DefaultFolder ? DEFAULT_FOLDER_ID : parent.getFullName());
+                                isParentDefault ? DEFAULT_FOLDER_ID : parent.getFullName());
                         }
                     } catch (final MessagingException e) {
                         if ("Unsupported type".equals(e.getMessage())) {
@@ -557,7 +569,7 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                                     session,
                                     e,
                                     createMe.getFullName(),
-                                    parent instanceof DefaultFolder ? DEFAULT_FOLDER_ID : parent.getFullName());
+                                    isParentDefault ? DEFAULT_FOLDER_ID : parent.getFullName());
                             }
                             if (LOG.isInfoEnabled()) {
                                 LOG.info("IMAP folder created with fallback type HOLDS_MESSAGES");
@@ -744,8 +756,15 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                     if (getChecker().isDefaultFolder(oldFullname)) {
                         throw IMAPException.create(IMAPException.Code.NO_DEFAULT_FOLDER_UPDATE, imapConfig, session, oldFullname);
                     }
-                    IMAPFolder destFolder =
-                        ((IMAPFolder) (MailFolder.DEFAULT_FOLDER_ID.equals(newParent) ? imapStore.getDefaultFolder() : imapStore.getFolder(newParent)));
+                    IMAPFolder destFolder;
+                    final boolean isDestRoot;
+                    if (MailFolder.DEFAULT_FOLDER_ID.equals(newParent)) {
+                        destFolder = (IMAPFolder) imapStore.getDefaultFolder();
+                        isDestRoot = true;
+                    } else {
+                        destFolder = (IMAPFolder) imapStore.getFolder(newParent);
+                        isDestRoot = false;
+                    }
                     if (!destFolder.exists()) {
                         destFolder = checkForNamespaceFolder(newParent);
                         if (null == destFolder) {
@@ -755,65 +774,68 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                             throw IMAPException.create(IMAPException.Code.FOLDER_NOT_FOUND, imapConfig, session, newParent);
                         }
                     }
-                    if (!inferiors(destFolder)) {
-                        throw IMAPException.create(
-                            IMAPException.Code.FOLDER_DOES_NOT_HOLD_FOLDERS,
-                            imapConfig,
-                            session,
-                            destFolder.getFullName());
-                    }
-                    if (imapConfig.isSupportsACLs() && isSelectable(destFolder)) {
-                        try {
-                            if (!getACLExtension().canCreate(RightsCache.getCachedRights(destFolder, true, session, accountId))) {
-                                throw IMAPException.create(IMAPException.Code.NO_CREATE_ACCESS, imapConfig, session, newParent);
-                            }
-                        } catch (final MessagingException e) {
-                            /*
-                             * MYRIGHTS command failed for given mailbox
-                             */
-                            if (!imapConfig.getImapCapabilities().hasNamespace() || !NamespaceFoldersCache.containedInPersonalNamespaces(
-                                newParent,
-                                imapStore,
-                                true,
+                    synchronized (destFolder) {
+                        if (!inferiors(destFolder)) {
+                            throw IMAPException.create(
+                                IMAPException.Code.FOLDER_DOES_NOT_HOLD_FOLDERS,
+                                imapConfig,
                                 session,
-                                accountId)) {
+                                isDestRoot ? DEFAULT_FOLDER_ID : destFolder.getFullName());
+                        }
+                        if (imapConfig.isSupportsACLs() && isSelectable(destFolder)) {
+                            try {
+                                if (isDestRoot) {
+                                    if (!(RootSubfolderCache.canCreateSubfolders((DefaultFolder) destFolder, true, session, accountId).booleanValue())) {
+                                        throw IMAPException.create(
+                                            IMAPException.Code.NO_CREATE_ACCESS,
+                                            imapConfig,
+                                            session,
+                                            DEFAULT_FOLDER_ID);
+                                    }
+                                } else {
+                                    if (!getACLExtension().canCreate(RightsCache.getCachedRights(destFolder, true, session, accountId))) {
+                                        throw IMAPException.create(IMAPException.Code.NO_CREATE_ACCESS, imapConfig, session, newParent);
+                                    }
+                                }
+                            } catch (final MessagingException e) {
                                 /*
-                                 * No namespace support or given parent is NOT covered by user's personal namespaces.
+                                 * MYRIGHTS command failed for given mailbox
                                  */
-                                throw IMAPException.create(IMAPException.Code.NO_ACCESS, imapConfig, session, e, newParent);
-                            }
-                            if (DEBUG) {
-                                LOG.debug("MYRIGHTS command failed on namespace folder", e);
+                                if (!imapConfig.getImapCapabilities().hasNamespace() || !NamespaceFoldersCache.containedInPersonalNamespaces(
+                                    newParent,
+                                    imapStore,
+                                    true,
+                                    session,
+                                    accountId)) {
+                                    /*
+                                     * No namespace support or given parent is NOT covered by user's personal namespaces.
+                                     */
+                                    throw IMAPException.create(IMAPException.Code.NO_ACCESS, imapConfig, session, e, newParent);
+                                }
+                                if (DEBUG) {
+                                    LOG.debug("MYRIGHTS command failed on namespace folder", e);
+                                }
                             }
                         }
-                    }
-                    final boolean mboxEnabled =
-                        MBoxEnabledCache.isMBoxEnabled(imapConfig.getImapServerSocketAddress(), destFolder, new StringBuilder(
-                            destFolder.getFullName()).append(separator).toString());
-                    if (!checkFolderNameValidity(newName, separator, mboxEnabled)) {
-                        throw IMAPException.create(
-                            IMAPException.Code.INVALID_FOLDER_NAME,
-                            imapConfig,
-                            session,
-                            Character.valueOf(separator));
-                    }
-
-                    if (destFolder.getFullName().startsWith(oldFullname)) {
-                        throw IMAPException.create(
-                            IMAPException.Code.NO_MOVE_TO_SUBFLD,
-                            imapConfig,
-                            session,
-                            moveMe.getName(),
-                            destFolder.getName());
-                    }
-                    try {
+                        final boolean mboxEnabled =
+                            MBoxEnabledCache.isMBoxEnabled(imapConfig.getImapServerSocketAddress(), destFolder, new StringBuilder(
+                                destFolder.getFullName()).append(separator).toString());
+                        if (!checkFolderNameValidity(newName, separator, mboxEnabled)) {
+                            throw IMAPException.create(
+                                IMAPException.Code.INVALID_FOLDER_NAME,
+                                imapConfig,
+                                session,
+                                Character.valueOf(separator));
+                        }
+                        if (destFolder.getFullName().startsWith(oldFullname)) {
+                            throw IMAPException.create(
+                                IMAPException.Code.NO_MOVE_TO_SUBFLD,
+                                imapConfig,
+                                session,
+                                moveMe.getName(),
+                                destFolder.getName());
+                        }
                         moveMe = moveFolder(moveMe, destFolder, newName);
-                    } catch (final MailException e) {
-                        deleteTemporaryCreatedFolder(destFolder, newName);
-                        throw e;
-                    } catch (final MessagingException e) {
-                        deleteTemporaryCreatedFolder(destFolder, newName);
-                        throw e;
                     }
                 }
                 /*
@@ -960,19 +982,17 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                             /*
                              * Default folder is affected, check if owner still holds full rights
                              */
+                            final ACLExtension aclExtension = getACLExtension();
                             if (getChecker().isDefaultFolder(updateMe.getFullName()) && !stillHoldsFullRights(
                                 updateMe,
                                 newACLs,
-                                imapConfig,
-                                session,
-                                ctx)) {
+                                aclExtension)) {
                                 throw IMAPException.create(
                                     IMAPException.Code.NO_DEFAULT_FOLDER_UPDATE,
                                     imapConfig,
                                     session,
                                     updateMe.getFullName());
                             }
-                            final ACLExtension aclExtension = getACLExtension();
                             if (!aclExtension.canSetACL(RightsCache.getCachedRights(updateMe, true, session, accountId))) {
                                 throw IMAPException.create(
                                     IMAPException.Code.NO_ADMINISTER_ACCESS,
@@ -1037,17 +1057,12 @@ public final class IMAPFolderStorage extends MailFolderStorage {
         }
     }
 
-    private void deleteTemporaryCreatedFolder(final IMAPFolder destFolder, final String name) throws MessagingException {
-        /*
-         * Delete moved folder if operation failed
-         */
-        final IMAPFolder tmp =
-            (IMAPFolder) imapStore.getFolder(new StringBuilder(destFolder.getFullName()).append(destFolder.getSeparator()).append(name).toString());
-        if (tmp.exists()) {
+    private void deleteTemporaryCreatedFolder(final IMAPFolder temporaryFolder) throws MessagingException {
+        if (temporaryFolder.exists()) {
             try {
-                tmp.delete(true);
+                temporaryFolder.delete(true);
             } catch (final MessagingException e1) {
-                LOG.error("Temporary created folder could not be deleted: " + tmp.getFullName(), e1);
+                LOG.error("Temporary created folder could not be deleted: " + temporaryFolder.getFullName(), e1);
             }
         }
     }
@@ -1100,14 +1115,16 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                                 (IMAPFolder) imapStore.getFolder(sb.append(trashFolder.getFullName()).append(deleteMe.getSeparator()).append(
                                     name).append('_').append(++appendix).toString());
                         }
-                        try {
-                            moveFolder(deleteMe, trashFolder, newFolder, false);
-                        } catch (final MailException e) {
-                            deleteTemporaryCreatedFolder(trashFolder, newFolder.getName());
-                            throw e;
-                        } catch (final MessagingException e) {
-                            deleteTemporaryCreatedFolder(trashFolder, newFolder.getName());
-                            throw e;
+                        synchronized (newFolder) {
+                            try {
+                                moveFolder(deleteMe, trashFolder, newFolder, false);
+                            } catch (final MailException e) {
+                                deleteTemporaryCreatedFolder(newFolder);
+                                throw e;
+                            } catch (final MessagingException e) {
+                                deleteTemporaryCreatedFolder(newFolder);
+                                throw e;
+                            }
                         }
                     }
                 }
@@ -1421,9 +1438,13 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                         if (!getACLExtension().canRead(myrights)) {
                             throw IMAPException.create(IMAPException.Code.NO_READ_ACCESS, imapConfig, session, fullname);
                         }
+                        /*-
+                         * TODO: Why check DELETE access when requesting quota?
+                         * 
                         if (!getACLExtension().canDeleteMailbox(myrights)) {
                             throw IMAPException.create(IMAPException.Code.NO_DELETE_ACCESS, imapConfig, session, fullname);
                         }
+                        */
                     }
                 } catch (final MessagingException e) {
                     throw IMAPException.create(IMAPException.Code.NO_ACCESS, imapConfig, session, e, fullname);
@@ -1572,9 +1593,7 @@ public final class IMAPFolderStorage extends MailFolderStorage {
         UserFlagsCache.removeUserFlags(deleteMe, session, accountId);
     }
 
-    private static final transient Rights FULL_RIGHTS = new Rights("lrswipcda");
-
-    private static boolean stillHoldsFullRights(final IMAPFolder defaultFolder, final ACL[] newACLs, final IMAPConfig imapConfig, final Session session, final Context ctx) throws AbstractOXException, MessagingException {
+    private boolean stillHoldsFullRights(final IMAPFolder defaultFolder, final ACL[] newACLs, final ACLExtension aclExtension) throws AbstractOXException, MessagingException {
         /*
          * Ensure that owner still holds full rights
          */
@@ -1583,8 +1602,9 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                 session.getUserId(),
                 ctx,
                 IMAPFolderConverter.getEntity2AclArgs(session, defaultFolder, imapConfig));
-        for (int i = 0; i < newACLs.length; i++) {
-            if (newACLs[i].getName().equals(ownerACLName) && newACLs[i].getRights().contains(FULL_RIGHTS)) {
+        final Rights fullRights = aclExtension.getFullRights();
+        for (final ACL newACL : newACLs) {
+            if (newACL.getName().equals(ownerACLName) && newACL.getRights().contains(fullRights)) {
                 return true;
             }
         }
@@ -1634,21 +1654,24 @@ public final class IMAPFolderStorage extends MailFolderStorage {
 
     private IMAPFolder moveFolder(final IMAPFolder toMove, final IMAPFolder destFolder, final String folderName, final boolean checkForDuplicate) throws MessagingException, MailException {
         final String destFullname = destFolder.getFullName();
-        StringBuilder sb = new StringBuilder();
-        if (destFullname.length() > 0) {
-            sb.append(destFullname).append(destFolder.getSeparator());
+        final IMAPFolder newFolder;
+        {
+            final StringBuilder sb = new StringBuilder();
+            if (destFullname.length() > 0) {
+                sb.append(destFullname).append(destFolder.getSeparator());
+            }
+            sb.append(folderName);
+            newFolder = (IMAPFolder) imapStore.getFolder(sb.toString());
         }
-        sb.append(folderName);
-        final IMAPFolder newFolder = (IMAPFolder) imapStore.getFolder(sb.toString());
-        sb = null;
         return moveFolder(toMove, destFolder, newFolder, checkForDuplicate);
     }
 
     private static final String[] ARGS_ALL = { "1:*" };
 
     private IMAPFolder moveFolder(final IMAPFolder toMove, final IMAPFolder destFolder, final IMAPFolder newFolder, final boolean checkForDuplicate) throws MessagingException, MailException {
+        final String destFullName = destFolder.getFullName();
         if (!inferiors(destFolder)) {
-            throw IMAPException.create(IMAPException.Code.FOLDER_DOES_NOT_HOLD_FOLDERS, imapConfig, session, destFolder.getFullName());
+            throw IMAPException.create(IMAPException.Code.FOLDER_DOES_NOT_HOLD_FOLDERS, imapConfig, session, destFullName);
         }
         final int toMoveType = toMove.getType();
         final String moveFullname = toMove.getFullName();
@@ -1701,7 +1724,7 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                 imapConfig,
                 session,
                 newFullname,
-                destFolder instanceof DefaultFolder ? DEFAULT_FOLDER_ID : destFolder.getFullName());
+                destFolder instanceof DefaultFolder ? DEFAULT_FOLDER_ID : destFullName);
         }
         /*
          * Apply original subscription status
