@@ -685,7 +685,7 @@ public final class IMAPMessageStorage extends IMAPFolderWorker implements IMailM
                  */
                 final long start = System.currentTimeMillis();
                 final Message[] msgs =
-                    IMAPCommandsCollection.getUnreadMessages(imapFolder, fields, sortField, getIMAPProperties().isFastFetch());
+                    IMAPCommandsCollection.getUnreadMessages(imapFolder, fields, sortField, order, getIMAPProperties().isFastFetch(), limit);
                 mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - start);
                 if ((msgs == null) || (msgs.length == 0) || limit == 0) {
                     return EMPTY_RETVAL;
@@ -1263,12 +1263,8 @@ public final class IMAPMessageStorage extends IMAPFolderWorker implements IMailM
              * Remove non user-alterable system flags
              */
             int flags = flagsArg;
-            if (((flags & MailMessage.FLAG_RECENT) > 0)) {
-                flags = flags ^ MailMessage.FLAG_RECENT;
-            }
-            if (((flags & MailMessage.FLAG_USER) > 0)) {
-                flags = flags ^ MailMessage.FLAG_USER;
-            }
+            flags &= ~MailMessage.FLAG_RECENT;
+            flags &= ~MailMessage.FLAG_USER;
             /*
              * Set new flags...
              */
