@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -89,7 +89,7 @@ public final class RootSubfolderCache {
         final SessionMailCache mailCache = SessionMailCache.getInstance(session, accontId);
         mailCache.get(entry);
         if (load && (null == entry.getValue())) {
-            entry.setValue(Boolean.valueOf(canCreateSubfolder(f)));
+            entry.setValue(canCreateSubfolder(f));
             mailCache.put(entry);
         }
         return entry.getValue();
